module ade_ari; 


{============================================================================}
{===  mpi_ari  ======   Analytic Differentiable Expression Arithmetic =======}
{====================   =        =              =          ===        =======}
{============================================================================}
{                                                                            }
{    File    :  ae_ari.p                                                     }
{                                                                            }
{=   Version :  1.1                                                          }
{                                                                            }
{=   Date    :  05/../..  ( first implementation )                           }
{=              ../../..  ( last change )                                    }
{                                                                            }
{    Language:  PASCAL-XSC, Vers. 3.0 or later                               }
{                                                                            }
{    Author  :  K. Petras                                                    }
{                                                                            }
{============================================================================}
{      Das Modul behandelt komplexe Rechtecke um Intervalle                  }
{      Es  rechnet gleichzeitig mit Intervallen und solchen                  }
{      zur reellen Achse symmetrischen Rechtecken und prueft                 }
{      jeweils, ob die Funktion dort analytisch ist.                         }
{============================================================================}

 use global i_ari, bi_ari,
    arith; 



global type analyticd = global record
rint, dint : interval;
cint	   : icom;
   a, d	   : boolean;
end;	   

global var rerror : boolean; 
   
global function construct(x	 : interval;
			  r1, r2 : real) : analyticd;
var h : interval;
begin
   h:= (sup(x)-inf(x))/2; 
   construct.rint := x;
   construct.dint := 1;
   construct.cint := icompl(x+h*intval(-abs(r1-1), abs(r1-1)),
			   sup(h)*abs(r2));
   construct.a    := true;
   construct.d    := true;
end; { construct }

global function construct(x    : interval;
			  r    : real) : analyticd;
var s :real ; 
begin
   s:= abs(r); 
   construct := construct(x, (s+>1/>s)/2, (s->1/>s)/2);
end; { construct }


global procedure write(var f : text;
		    x : analyticd);
begin
   writeln(f,'reell_intv = [', inf(x.rint):20,
	   ' , ',sup(x.rint):20,' ]');
   if (x.d) then
      writeln(f, 'abl_intv   = [', inf(x.dint):20,
	   ' , ',sup(x.dint):20,' ]')
   else
      writeln(f, 'keine Ableitungseinschliessung berechnet'); 
   if (x.a) then
      begin
	 writeln(f, 'compl_intv =   [', inf(x.cint.re):20,
		 ' , ',sup(x.cint.re):20,' ]');
	 writeln(f, '    + i   [', -x.cint.im:20,
		 ' , ',x.cint.im:20,' ]');
      end
   else
      writeln(f, 'keine komplexe Einschliessung berechnet'); 
end; { write }

{==========   Hilfsfunktionen ============}

global function dia (x	: icom):real;
begin
   if ((abs(x.re.sup) <1.0e150) and 
       (abs(x.re.inf) <1.0e150) and 
       (x.im <1.0e150)) then
      dia := 1.00000001*sqrt(diam(x.re)+ sqr(2*x.im))
   else
      dia := diam(x.re) + 2*x.im; 
end;
			
global function max (n, m : integer):integer;
begin
   if (n>m) then
      max:=n
   else
      max:=m;
end; { max }



{==========   Vorzeichenumkehr =================================}

global operator -  ( x : analyticd) ch : analyticd;
begin
   ch.rint:=-x.rint;
   ch.dint:=-x.dint;
   ch.cint.re:=-x.cint.re;
   ch.cint.re:=x.cint.im;
   ch.a:=x.a;
   ch.d:=x.d;
end;


{==========   Addition mit analytischer Beteiligung ============}

global operator +  ( x,y : analyticd) add : analyticd;
begin
   if (rerror) then
      add :=construct(intval(0), 0.0, 0.0)
   else
   begin
      if (x.d and y.d) then
      begin
	 add.dint :=x.dint + y.dint; 
	 add.d    := true;
      end
      else
      begin
	 add.dint :=0; 
	 add.d    := false;
      end;
      if (x.a and y.a) then
      begin
	 add.rint := x.rint + y.rint; 
	 add.cint := x.cint + y.cint; 
	 add.a    := true;
      end
      else
      begin
	 add.rint := x.rint + y.rint; 
	 add.cint := icompl(intval(0)); 
	 add.a    := false;
      end;
   end;
end; { add }



global operator +  (x : analyticd;
		    y : interval ) add : analyticd;
begin
if (rerror) then
   add :=construct(intval(0), 0.0, 0.0)
else
   begin
      if (x.d) then
      begin
	 add.dint :=x.dint ; 
	 add.d    := true;
      end
      else
      begin
	 add.dint :=0; 
	 add.d    := false;
      end;
      if (x.a) then
      begin
	 add.rint := x.rint + y; 
	 add.cint := x.cint + y; 
	 add.a    := true;
      end
      else
      begin
	 add.rint := x.rint + y; 
	 add.cint := icompl(intval(0)); 
	 add.a    := false;
      end;
   end;
end; { add }

      

global operator +  (x : interval;
		    y : analyticd ) add : analyticd;
begin
if (rerror) then
   add :=construct(intval(0), 0.0, 0.0)
else
   begin
      if (y.d) then
      begin
	 add.dint :=y.dint ; 
	 add.d    := true;
      end
      else
      begin
	 add.dint :=0; 
	 add.d    := false;
      end;
      if (y.a) then
      begin
	 add.rint := x + y.rint; 
	 add.cint := x + y.cint; 
	 add.a    := true;
      end
      else
      begin
	 add.rint :=  + y.rint; 
	 add.cint := icompl(intval(0)); 
	 add.a    := false;
      end;
   end;
end; { add }



global operator +  (x : real;
		    y : analyticd ) add : analyticd;
begin
   add:=intval(x)+y;
end; { ade_neu }

global operator +  (x : integer;
		    y : analyticd ) add : analyticd;
begin
   add:=intval(x)+y;
end;

global operator +  ( y : analyticd; x : real) add : analyticd;
begin
   add:=intval(x)+y;
end; { ade_neu }

global operator +  ( y : analyticd; x : integer) add : analyticd;
begin
   add:=intval(x)+y;
end; { ade_neu }



{==========   Subtraktion mit analytischer Beteiligung ============}

global operator -  ( x,y : analyticd) diff : analyticd;
begin
if (rerror) then
   diff :=construct(intval(0), 0.0, 0.0)
else
   begin
      if (x.d and y.d) then
      begin
	 diff.dint := x.dint-y.dint ; 
	 diff.d    := true;
      end
      else
      begin
	 diff.dint :=0; 
	 diff.d    := false;
      end;
      if (x.a and y.a) then
      begin
	 diff.rint := x.rint - y.rint; 
	 diff.cint := x.cint - y.cint; 
	 diff.a    := true;
      end
      else
      begin
	 diff.rint := x.rint - y.rint; 
	 diff.cint := icompl(intval(0)); 
	 diff.a    := false;
      end;
   end;
end; { diff }



global operator -  (x : analyticd;
		    y : interval ) diff : analyticd;
begin
if (rerror) then
   diff :=construct(intval(0), 0.0, 0.0)
else
   begin
      if (x.d) then
      begin
	 diff.dint := x.dint ; 
	 diff.d    := true;
      end
      else
      begin
	 diff.dint :=0; 
	 diff.d    := false;
      end;
   if (x.a) then
   begin
      diff.rint := x.rint - y; 
      diff.cint := x.cint - y; 
      diff.a    := true;
   end
   else
   begin
      diff.rint := x.rint - y; 
      diff.cint := icompl(intval(0)); 
      diff.a    := false;
   end;
   end;
end; { diff }


global operator -  (x : interval;
		    y : analyticd ) diff : analyticd;
begin
if (rerror) then
   diff :=construct(intval(0), 0.0, 0.0)
else
   begin
      if (y.d) then
      begin
	 diff.dint := -y.dint ; 
	 diff.d    := true;
      end
      else
      begin
	 diff.dint :=0; 
	 diff.d    := false;
      end;
   if (y.a) then
   begin
      diff.rint := x - y.rint; 
      diff.cint := x - y.cint; 
      diff.a    := true;
   end
   else
   begin
      diff.rint := x - y.rint; 
      diff.cint := icompl(intval(0)); 
      diff.a    := false;
   end;
   end;
end; { diff }

      
global operator -  (x : real;
		    y : analyticd ) sub : analyticd;
begin
   sub:=intval(x)-y;
end; { ade_neu }

operator -  (x : integer;  y : analyticd ) sub : analyticd;
begin
   sub:=intval(x)-y;
end;

global operator -  ( y : analyticd; x : real) sub : analyticd;
begin
   sub:=y-intval(x);
end; { ade_neu }

global operator -  ( y : analyticd; x : integer) sub : analyticd;
begin
   sub:=y-intval(x);
end; { ade_neu }


{==========   Multiplikation mit analytischer Beteiligung ============}

global operator *  ( x,y : analyticd) prod : analyticd;
begin
if (rerror) then
   prod :=construct(intval(0), 0.0, 0.0)
else
   begin
      if (x.d and y.d) then
      begin
	 prod.dint := x.dint*y.rint + x.rint*y.dint; 
	 prod.d    := true;
      end
      else
      begin
	 prod.dint :=0; 
	 prod.d    := false;
      end;
   if (x.a and y.a) then
   begin
      if (max(expo(sup(abs(x.cint.re))), 0)
	      + max(expo(x.cint.im), 0)
	      + max(expo(sup(abs(y.cint.re))), 0)
	      +  max(expo(y.cint.im), 0) <1020) then
      begin
	 prod.rint := x.rint * y.rint; 
	 prod.cint := x.cint*y.cint; 
	 prod.a    := true;
      end
      else
      begin
	 prod.rint := x.rint * y.rint; 
	 prod.cint := icompl(intval(0)); 
	 prod.a    := false;
      end
   end
   else
   begin
      prod.rint := x.rint * y.rint; 
      prod.cint := icompl(intval(0)); 
      prod.a    := false;
   end;
   end;
end; { prod }



global operator *  (x : analyticd;
		    y : interval ) prod : analyticd;
begin
if (rerror) then
   prod :=construct(intval(0), 0.0, 0.0)
else
   begin
      if (x.d) then
      begin
	 prod.dint := x.dint*y     ; 
	 prod.d    := true;
      end
      else
      begin
	 prod.dint :=0; 
	 prod.d    := false;
      end;
   if (x.a) then
   begin
      prod.rint := x.rint * y; 
      prod.cint := x.cint*y; 
      prod.a    := true;
   end
   else
   begin
      prod.rint := x.rint * y; 
      prod.cint := icompl(intval(0)); 
      prod.a    := false;
   end;
   end;
end; { prod }

      

global operator *  (x : interval;
		    y : analyticd ) prod : analyticd;
begin
if (rerror) then
   prod :=construct(intval(0), 0.0, 0.0)
else
begin
   if (y.d) then
   begin
      prod.dint := x *y.dint; 
      prod.d    := true;
   end
   else
   begin
	 prod.dint :=0; 
      prod.d    := false;
   end;
   if (y.a) then
   begin
      prod.rint := x * y.rint; 
      prod.cint := y.cint * x;  
      prod.a    := true;
   end
   else
   begin
      prod.rint := x * y.rint; 
      prod.cint := icompl(intval(0)); 
      prod.a    := false;
   end;
end;
end; { prod }

global operator *  (x : real;
		    y : analyticd ) mul : analyticd;
begin
   mul:=intval(x)*y;
end; { ade_neu }

 operator *  (x : integer;
		    y : analyticd ) mul : analyticd;
begin
   mul:=intval(x)*y;
end;

global operator *  ( y : analyticd; x : real) mul : analyticd;
begin
   mul:=y*intval(x);
end; { ade_neu }

global operator *  ( y : analyticd; x : integer) mul : analyticd;
begin
   mul:=y*intval(x);
end; { ade_neu }

{==========   Division mit analytischer Beteiligung ============}

global operator /  ( x,y : analyticd) quot : analyticd;
var xdy: interval; 
begin
if (rerror) then
   quot :=construct(intval(0), 0.0, 0.0)
else
   begin
   if (0 in y.rint) then
   begin
      quot := construct(intval(0.0), 0.0, 0.0);
      rerror := true;
   end
   else
   begin
      if (x.a and y.a) then
      begin
	 if (0 in y.cint.re) then
	 begin 
	    xdy := x.rint / y.rint; 
	    quot.cint := icompl(intval(0)); 
	    quot.a    := false;
	 end
	 else
	 begin
	    xdy := x.rint / y.rint; 
	    quot.cint :=x.cint/y.cint;
	    quot.a    := true;
	 end;
      end
      else
	 begin 
	    xdy := x.rint / y.rint; 
	    quot.cint := icompl(intval(0)); 
	    quot.a    := false;
	 end;
      quot.rint := xdy; 
      if (x.d and y.d) then
      begin
	 quot.dint := (x.dint-y.dint*xdy)/y.rint; 
	 quot.d    := true;
      end
      else
      begin
	 quot.dint :=0; 
	 quot.d    := false;
      end;
   end;      
   end;
end; { quot }


global operator /  (x : analyticd;
		    y : interval ) quot : analyticd;
var edy	: interval;
begin
if (rerror) then
   quot :=construct(intval(0), 0.0, 0.0)
else
   begin
   if (0 in y) then
   begin
      quot := construct(intval(0.0), 0.0, 0.0);
      rerror := true;
   end
   else
   begin
      edy:=1/y;
      if (x.d) then
      begin
	 quot.dint := edy*x.dint; 
	 quot.d    := true;
      end
      else
      begin
	 quot.dint :=0; 
	 quot.d    := false;
      end;
      if (x.a) then
      begin
	 quot.rint := edy*x.rint ; 
	 quot.cint := icompl(edy*x.cint.re,  sup(abs(edy))*>x.cint.im); 
	 quot.a    := true;
      end
      else
	 begin 
	    quot.rint := x.rint / y; 
	    quot.cint := icompl(intval(0)); 
	    quot.a    := false;
	 end;
   end;
   end;
end; { quot }

global operator /  (x : interval;
		    y : analyticd) quot : analyticd;
var c, faktor : interval;
   d	      : real;
begin
if (rerror) then
   quot :=construct(intval(0), 0.0, 0.0)
else
   begin
   if (0 in y.rint) then
   begin
      quot := construct(intval(0.0), 0.0, 0.0);
      rerror := true;
   end
   else
   begin
      if (y.d) then
      begin
	 quot.dint := -x*y.dint/sqr(y.rint); 
	 quot.d    := true;
      end
      else
      begin
	 quot.dint :=0; 
	 quot.d    := false;
      end;
      if ( y.a) then
      begin
	 if (0 in y.cint.re) then
	 begin 
	    quot.rint := x / y.rint; 
	    quot.cint := icompl(intval(0)); 
	    quot.a    := false;
	 end
	 else
	 begin
	    quot.rint := x / y.rint; 
	    c:=re(y.cint);
	    d:=im(y.cint);
	    faktor:=x/(sqr(c)+intval(0,d*>d));
	    quot.cint := icompl(faktor*c, sup(abs(faktor))*>d); 
	    quot.a    := true;
	 end;
      end
      else
	 begin 
	    quot.rint := x / y.rint; 
	    quot.cint := icompl(intval(0)); 
	    quot.a    := false;
	 end;
   end;     
   end; 
end; { quot }


global operator /  (x : real;
		    y : analyticd ) quot : analyticd;
begin
   quot:=intval(x)/y;
end; { ade_neu }

operator /  (x : integer;
		    y : analyticd ) quot : analyticd;
begin
   quot:=intval(x)/y;
end;

global operator /  ( y : analyticd; x : real) quot : analyticd;
begin
   quot:=y/intval(x);
end; { ade_neu }

global operator /  ( y : analyticd; x : integer) quot : analyticd;
begin
   quot:=y/intval(x);
end; { ade_neu }


{==========   Standardfunktionen :     ======================}

global function sqr (x	: analyticd) : analyticd;
var xq	: interval;
   hilf	: integer; 
begin
if (rerror) then
   sqr :=construct(intval(0), 0.0, 0.0)
else
   begin
      if (max(expo(sup(x.rint)), expo(inf(x.rint))) >510) then
      begin
	 sqr :=construct(intval(0), 0.0, 0.0);
	 rerror := true;
      end
      else
      begin
	 xq := sqr(x.rint); 
	 sqr.rint := xq;
	 hilf := max(expo(sup(re(x.cint))),expo(inf(re(x.cint)))); 
	 hilf := max(hilf, expo(im(x.cint))); 
	 if ((hilf <510) and x.a) then
	 begin
	    sqr.cint := sqr(x.cint); 
	    sqr.a    := true;
	 end
	 else
	 begin
	    sqr.cint := icompl(intval(0)); 
	    sqr.a    := false;
	 end;
	 if (x.d) then
	 begin
	    sqr.dint := 2*x.dint * x.rint;
	    sqr.d    := true
	 end
	 else
	 begin
	    sqr.dint :=0; 
	    sqr.d    := false;
	 end;
      end;
   end;
end; { sqr }


global function pow(x : analyticd;
		    n : integer) : analyticd; 
var xq	    : interval;
   hilf	    : real;
   k, i, si : integer;
   pot2	    : array[1..1030] of icom; 
   prod	    : icom; 
begin
if (rerror) then
   pow :=construct(intval(0), 0.0, 0.0)
else
   if (n<0) then
      pow:=intval(1.0)/pow(x, -n)
   else
   begin
      if (n=0) then
	 if (0 in x.rint) then
	 begin
	    rerror := true; 
	    pow :=construct(intval(0), 0.0, 0.0)
	 end
	 else
	 begin
	    pow.rint := intval(1);
	    pow.dint := intval(0);
	    pow.d    := true; 
	    if (x.a) then
	    begin
	       if (0 in x.cint.re) then
	       begin
		  pow.cint := icompl(intval(0)); 
		  pow.a := false;
	       end		  
	       else
	       begin
		  pow.cint := icompl(intval(1)); 
		  pow.a := true;
	       end;
	    end;
	 end
      else
      begin
	 if (n*max(expo(sup(x.rint)), expo(inf(x.rint))) >1020) then
	 begin
	    pow :=construct(intval(0), 0.0, 0.0);
	    rerror := true;
	 end
	 else
	 begin
	    if (n=1) then  pow := x;
	    if (n=2) then pow:=sqr(x);
	    if (n>=3) then
	    begin
	       pow.rint := pow(x.rint, n);
	       if (x.d) then
	       begin
		  pow.dint := n*x.dint*pow(x.rint, n-1);
		  pow.d := true
	       end
	       else
	       begin
		  pow.dint:=intval(0);
		  pow.d :=false;
	       end; 
	       if (x.a and (n*expo(sup(abs(x.cint))) <1000)) then
	       begin   
		  k:=2;
		  i:=1;
		  pot2[1] := x.cint;
		  while (k<=n) do
		  begin
		     i:=i+1; 
		     pot2[i]:=sqr(pot2[i-1]);
		     k:=2*k;
		  end;
		  k:=k div 2;
		  prod:=pot2[i];
		  si:=k; 
		  while (si<n) do
		  begin
		     k:=k div 2;
		     i:=i-1; 
		     if (si+k<=n) then
		     begin
			prod:=prod*pot2[i];
			si:=si+k;
		     end;
		  end;
		  pow.cint:=prod;
		  pow.a:=true; 
	       end
	       else
	       begin
		  pow.cint := icompl(intval(0)); 
		  pow.a := false;
	       end;		  
	    end; 
	 end;
      end;
   end;
end; { pow }


global function pospart ( x : analyticd) : analyticd;
begin
if (rerror) then
   pospart :=construct(intval(0), 0.0, 0.0)
else
   begin
      if (0 >= sup(x.rint)) then
	 pospart :=construct(intval(0), 0.0, 0.0)
      else
      begin
	 if (0 <= inf(x.rint)) then
	    pospart:=x
	 else
	 begin
	    pospart.rint :=intval(0.0, sup(x.rint)); 
	    pospart.dint := 0.0 +* x.dint;
	    pospart.d    := x.d;
	    pospart.cint := icompl(intval(0)); 
	    pospart.a    := false;
	 end;
      end;
   end;
end; { pospart }

global function exp (x	: analyticd) : analyticd;
var ehx	: interval; 
begin
if (rerror) then
   exp :=construct(intval(0), 0.0, 0.0)
else
   begin
      if (sup(x.rint)>709) then
      begin
	 exp :=construct(intval(0), 0.0, 0.0);
	 rerror := true;
      end
      else
      begin
	 ehx := exp(x.rint); 
	 exp.rint := ehx; 
	 if ((sup(re(x.cint))<=709) and x.a) then
	 begin
	    exp.cint := exp(x.cint); 
	    exp.a    := true;
	 end
	 else
	 begin
	    exp.cint := icompl(intval(0)); 
	    exp.a    := false;
	 end;
	 if (x.d) then
	 begin
	    exp.dint := x.dint * ehx;
	    exp.d    := true
	 end
	 else
	 begin
	    exp.dint :=0; 
	    exp.d    := false;
	 end;
      end;
   end;
end; { exp }

global function sin (x	: analyticd) : analyticd;
var sx, cx : interval; 
begin
if (rerror) then
   sin :=construct(intval(0), 0.0, 0.0)
else
   begin
      sx := sin(x.rint); 
      cx := cos(x.rint); 
      sin.rint := sx; 
      if ((x.cint.im<=709) and x.a) then
      begin
	 sin.cint := sin(x.cint); 
	 sin.a    := true;
      end
      else
      begin
	 sin.cint := icompl(intval(0)); 
	 sin.a    := false;
      end;
      if (x.d) then
      begin
	 sin.dint := x.dint * cx;
	 sin.d    := true
      end
      else
      begin
	 sin.dint :=0; 
	 sin.d    := false;
      end;
   end;
end; { sin }

global function cos (x	: analyticd) : analyticd;
var sx, cx : interval; 
begin
   if (rerror) then
      cos :=construct(intval(0), 0.0, 0.0)
   else
   begin
      sx := sin(x.rint); 
      cx := cos(x.rint); 
      cos.rint := cx; 
      if ((x.cint.im<=709) and x.a) then
      begin
	 cos.cint := cos(x.cint); 
      cos.a    := true;
      end
   else
   begin
      cos.cint := icompl(intval(0)); 
      cos.a    := false;
   end;
      if (x.d) then
      begin
	 cos.dint := -x.dint * sx;
	 cos.d    := true
      end
      else
      begin
	 cos.dint :=0; 
	 cos.d    := false;
      end;
   end;
end; { cos }



global function sqrtabs (x : analyticd) : analyticd;
var   wurzel : interval; 
begin
   if (rerror) then
      sqrtabs :=construct(intval(0), 0.0, 0.0)
   else
   begin
      wurzel := sqrt(abs(x.rint));
      sqrtabs.rint := wurzel; 
      if (x.a or x.d) then
      begin
	 if (sup(x.rint)<0.0) then
	 begin
	    if (x.d) then
	    begin
	       sqrtabs.dint := -x.dint/(2*wurzel);
	       sqrtabs.d    := true;
	    end
	    else
	    begin
	       sqrtabs.dint := intval(0); 
	       sqrtabs.d    := false;
	    end; 
	    if (not(x.a)) then
	    begin
	       sqrtabs.cint := icompl(intval(0));
	       sqrtabs.a    := false;
	    end
	    else
	    begin
	       if (sup(re(x.cint))<0.0) then
	       begin
		  sqrtabs.cint := sqrt(-x.cint); 
		  sqrtabs.a    := true
	       end
	       else
	       begin
		  sqrtabs.cint := icompl(intval(0));
		  sqrtabs.a    := false;
	       end
	    end
	 end
	 else
	 begin
	    if (inf(x.rint)>0.0) then
	    begin
	       if (x.d) then
	       begin
		  sqrtabs.dint := x.dint/(2*wurzel);
		  sqrtabs.d    := true;
	       end
	       else
	       begin
		  sqrtabs.dint := intval(0); 
		  sqrtabs.d    := false;
	       end; 
	       if (not(x.a)) then
	       begin
		  sqrtabs.cint := icompl(intval(0));
		  sqrtabs.a    := false;
	       end
	       else
	       begin
		  if (inf(re(x.cint))>0.0) then
		  begin
		     sqrtabs.cint := sqrt(x.cint); 
		     sqrtabs.a    := true
		  end
		  else
		  begin
		     sqrtabs.cint := icompl(intval(0));
		     sqrtabs.a    := false;
		  end
	       end
	    end
	    else {0 liegt in jedem Argumentbereich}
	    begin
	       sqrtabs.dint := intval(0); 
	       sqrtabs.d    := false;
	       sqrtabs.cint := icompl(intval(0));
	       sqrtabs.a    := false
	    end
	 end
      end
   end;
end; { sqrtabs }
   

global function abs (x	: analyticd) : analyticd;
var abx	: interval; 
begin
   if (rerror) then
      abs :=construct(intval(0), 0.0, 0.0)
   else
   begin
      abx := abs(x.rint); 
      abs.rint := abx;
      if (x.a or x.d) then
      begin
	 if (sup(x.rint)<=0.0) then
	 begin
	    if (x.a) then
	    begin
	       abs.a:=true;
	       abs.cint := - x.cint
	    end
	    else
	    begin
	       abs.cint := icompl(intval(0)); 
	       abs.a    := false;
	    end;
	    if (x.d) then
	    begin
	       abs.d:=true;
	       abs.dint := - x.dint
	    end
	    else
	    begin
	       abs.dint := intval(0); 
	       abs.d    := false;
	    end;
	 end
	 else
	 begin
	    if (inf(x.rint)>=0.0) then
	    begin
	       if (x.a) then
	       begin
		  abs.a:=true;
		  abs.cint :=   x.cint
	       end
	       else
	       begin
		  abs.cint := icompl(intval(0)); 
		  abs.a    := false;
	       end;
	       if (x.d) then
	       begin
		  abs.d:=true;
		  abs.dint :=   x.dint
	       end
	       else
	       begin
		  abs.dint := intval(0); 
		  abs.d    := false;
	       end;
	    end
	    else
	    begin
	       abs.a := false;
	       if (x.d) then
	       begin
		  abs.d := true;
		  abs.dint := (-x.dint)+* x.dint 
	       end
	       else
	       begin
		  abs.dint := intval(0); 
		  abs.d    := false;
	       end
	    end;
	 end;
      end
      else
      begin 
	 abs.cint := icompl(intval(0)); 
	 abs.a    := false;
	 abs.dint := intval(0); 
	 abs.d    := false;
      end;
   end;
end; { abs }
   
   


end.

